##########################################################################################################
## Program: buildRegentsExamScores
## Description: builds regents exam outcomes and dates
##########################################################################################################

print("Building Regents exam dataset")

##########################################################################################################
## NOTES

## 1) .dta are converted to .Rds and saved, if user does not wish to re-read in .dta, set dt.flag == 0
## 2) .Rds are read in and stacked, if user does not wish to re-stack and save, set rds.stack.flag == 0

dta.flag <- 0
rds.stack.flag <- 0

##########################################################################################################
## Import

## 2004 - 2008 Cohort Based Files

if (dta.flag == 1) {
  
  raw.regents.04 <- read_dta(paste0(dataDir, "/regentsScores/y2004_regents_4_15.dta"))
  dt.raw.regents.04 <- as.data.table(raw.regents.04)
  saveRDS(dt.raw.regents.04, file = paste0(cleanDir, "/dt_raw_regents_2004.Rds"))
  rm(list = c("raw.regents.04", "dt.raw.regents.04"))
  
  raw.regents.05 <- read_dta(paste0(dataDir, "/regentsScores/y2005_regents_4_15.dta"))
  dt.raw.regents.05 <- as.data.table(raw.regents.05)
  saveRDS(dt.raw.regents.05, file = paste0(cleanDir, "/dt_raw_regents_2005.Rds"))
  rm(list = c("raw.regents.05", "dt.raw.regents.05"))
  
  raw.regents.06 <- read_dta(paste0(dataDir, "/regentsScores/y2006_regents_4_15.dta"))
  dt.raw.regents.06 <- as.data.table(raw.regents.06)
  saveRDS(dt.raw.regents.06, file = paste0(cleanDir, "/dt_raw_regents_2006.Rds"))
  rm(list = c("raw.regents.06", "dt.raw.regents.06"))
  
  raw.regents.07 <- read_dta(paste0(dataDir, "/regentsScores/y2007_regents_4_15.dta"))
  dt.raw.regents.07 <- as.data.table(raw.regents.07)
  saveRDS(dt.raw.regents.07, file = paste0(cleanDir, "/dt_raw_regents_2007.Rds"))
  rm(list = c("raw.regents.07", "dt.raw.regents.07"))
  
  raw.regents.08 <- read_dta(paste0(dataDir, "/regentsScores/y2008_regents_4_15.dta"))
  dt.raw.regents.08 <- as.data.table(raw.regents.08)
  saveRDS(dt.raw.regents.08, file = paste0(cleanDir, "/dt_raw_regents_2008.Rds"))
  rm(list = c("raw.regents.08", "dt.raw.regents.08"))
  
}

if (rds.stack.flag == 1) {

  dt.raw.04 <- readRDS(paste0(cleanDir, "/dt_raw_regents_2004.Rds"))
  dt.raw.05 <- readRDS(paste0(cleanDir, "/dt_raw_regents_2005.Rds"))
  dt.raw.06 <- readRDS(paste0(cleanDir, "/dt_raw_regents_2006.Rds"))
  dt.raw.07 <- readRDS(paste0(cleanDir, "/dt_raw_regents_2007.Rds"))
  dt.raw.08 <- readRDS(paste0(cleanDir, "/dt_raw_regents_2008.Rds"))
  
  dt.stack.0408.all <- rbind(dt.raw.04, dt.raw.05, dt.raw.06, dt.raw.07, dt.raw.08, fill=TRUE)
  
  rm(list = c("dt.raw.04","dt.raw.05","dt.raw.06","dt.raw.07","dt.raw.08"))
  gc()
  
  # retain unique observations (dropping file year)
  dt.stack.0408 <- unique(dt.stack.0408.all[, setdiff(colnames(dt.stack.0408.all), "year"), with = FALSE])
  
  saveRDS(dt.stack.0408, file = paste0(cleanDir, "dt_raw_regents_0408_stack.Rds"))
  
} else {
  
  dt.stack.0408 <- readRDS(paste0(cleanDir, "/dt_raw_regents_0408_stack.Rds"))

}

## 2009 Test-Based Raw Files

raw.regents.09 <- read_dta(paste0(dataDir, "/regentsScores/y2009_regents_all_4_16_19.dta"))
dt.raw.regents.09 <- as.data.table(raw.regents.09)

rm(list = c("raw.regents.09"))
gc()

########################################################################################################
## Cleaning

dt.stack.0408[, sid := as.numeric(NEWID)]

setkey(dt.stack.0408, sid)

# function that take last score within a given year
regents.pre09.nodup.score <- function(dt, score.name, date.name){
  
  dt[, temp_score := dt[, score.name, with = FALSE]]
  dt[, temp_date := dt[, date.name, with = FALSE]]
  
  # subset to non-missing scores
  dt.score.nonmiss <- unique(dt[temp_score!="", .(sid, temp_score, temp_date)])
  
  # extract month/year/day
  dt.score.nonmiss[temp_date != "", test.month := as.numeric(substr(temp_date, 1, 2))]
  dt.score.nonmiss[temp_date != "", test.day := as.numeric(substr(temp_date, 4, 5))]
  dt.score.nonmiss[temp_date != "", test.year := as.numeric(substr(temp_date, 7, 10))]
  
  setkey(dt.score.nonmiss, sid, test.year, test.month, test.day)
  
  # set academic year (identified by Spring year)
  dt.score.nonmiss[test.month %in% seq(9,12), academic.year := test.year + 1]
  dt.score.nonmiss[test.month %in% seq(1,8), academic.year := test.year]
  
  ## Take Latest Score within an Academic Year
  
  # flag duplicates
  dt.score.nonmiss[, count := .N, by = .(sid, academic.year)]
  
  # split into unique and duplicate
  dt.score.nonmiss.unique <- dt.score.nonmiss[count == 1]
  dt.score.nonmiss.dup <- dt.score.nonmiss[count > 1]
  
  if (nrow(dt.score.nonmiss.dup) > 0) {

    dt.score.nonmiss.dup[, temp_score_latest := .SD[.N,temp_score], by = .(sid, academic.year)]
  
  }
  
  dt.score.nonmiss.unique[, temp_score_latest := temp_score]
  
  # stack
  dt.score.nonmiss.nodup <- rbind(dt.score.nonmiss.unique, dt.score.nonmiss.dup, fill = TRUE)[, .(sid,academic.year, temp_score_latest)]
  
  dt.score.nonmiss.nodup[, (paste0(score.name)) := temp_score_latest]
  dt.score.nonmiss.nodup[, temp_score_latest := NULL]
  
  dt[, temp_score := NULL]
  dt[, temp_date := NULL]
  
  rm(list = c("dt.score.nonmiss", "dt.score.nonmiss.unique", "dt.score.nonmiss.dup"))
  gc()
  
  return(dt.score.nonmiss.nodup)
}

# extract scores and test dates
score.list <- grep("SCORE", colnames(dt.stack.0408), value = TRUE)
date.list <- gsub("SCORE", "TDATE", score.list)

# by test, return score by year
dt.regents.0408.list <- mapply(regents.pre09.nodup.score, score.list, date.list, MoreArgs = list(dt = dt.stack.0408), SIMPLIFY = FALSE)

# merge by year
dt.regents.cohort <- Reduce(function(...) merge(..., by = c("sid", "academic.year"), all = TRUE), dt.regents.0408.list)

# drop inappropriate dates
dt.regents.cohort.valid.date <- dt.regents.cohort[!is.na(academic.year) & academic.year %in% seq(2004,2008)]

# lowercase column names
colnames(dt.regents.cohort.valid.date) <- tolower(colnames(dt.regents.cohort.valid.date))

# rename select scores
setnames(dt.regents.cohort.valid.date, old = c("hscore", "fscore"), new = c("uscore", "flscore"), skip_absent = TRUE)

rm(list = c("dt.regents.0408.list", "dt.regents.cohort", "dt.stack.0408"))
gc()

########################################################################################################
## Cleaning 2009 Exam File

dt.raw.regents.09[, sid := as.numeric(newid)]

setkey(dt.raw.regents.09, sid)

# identify from item name and exam code
dt.raw.regents.09[item_name == "ELA REG" | exam_code %in% c("EXRL"), test.name := "escore"]
dt.raw.regents.09[item_name == "GLOBHISTRG" | exam_code %in% c("HXR$"), test.name := "gscore"]
dt.raw.regents.09[item_name == "PHSET ES R" | exam_code %in% c("SXRU"), test.name := "esscore"]
dt.raw.regents.09[item_name == "LIVENVIR R" | exam_code %in% c("SXRK"), test.name := "bscore"]
dt.raw.regents.09[item_name == "GEOMETRY REG" | exam_code %in% c("MXRG"), test.name := "geoscore"]
dt.raw.regents.09[item_name == "PHSETPHYSR" | exam_code %in% c("SXR$"), test.name := "pscore"]
dt.raw.regents.09[item_name == "PHSETCHEMR" | exam_code %in% c("SXRX"), test.name := "cscore"]
dt.raw.regents.09[item_name == "MATH A REG" | exam_code %in% c("MXRA"), test.name := "mscore"]
dt.raw.regents.09[item_name == "USHIST REG" | exam_code %in% c("HXRA"), test.name := "uscore"]
dt.raw.regents.09[item_name == "USHGOV REG" | exam_code %in% c("HXRU"), test.name := "uscore"]
dt.raw.regents.09[item_name == "ALGEBRA REG" | exam_code %in% c("MXRE","MXREE"), test.name := "algscore"]
dt.raw.regents.09[item_name == "ALG2 TRIG REG" | exam_code %in% c("MXRT"), test.name := "alg2trigscore"]
dt.raw.regents.09[exam_code %in% c("MXRB"), test.name := "mbscore"]
dt.raw.regents.09[item_name %in% c("FRENCH REG","HEBREW REG","LATIN REG","POLISH REG","SPAN REG","ITAL REG","GREEK REG") | 
  exam_code %in% c("FXRF","FXRH","FXRL","FXRN","FXRS","FXRT","FXRE"), test.name := "flscore"]

# compute maximum score if multiple tests taken in a year
dt.raw.regents.09[!is.na(test.name), score.max := as.numeric(max(score)), by = .(sid, year, test.name)]

dt.regents.09.w <- dcast(unique(dt.raw.regents.09[!is.na(test.name), c("sid","year","score.max","test.name")]), sid + year ~ test.name, value.var = c("score.max"))

dt.regents.09.w[, academic.year := 2009]

########################################################################################################
## Combine

# stack 2009
dt.regents.0409 <- rbind(dt.regents.cohort.valid.date, dt.regents.09.w, fill = TRUE)

setkey(dt.regents.0409, sid, academic.year)

rm(list = c("dt.raw.regents.09", "dt.regents.cohort.valid.date", "dt.regents.09.w"))
gc()

##########################################################################################################
## Compute Regents Outcomes

setDT(dt.regents.0409)

exams.to.compute <- c("escore","bscore","cscore","esscore","flscore","pscore","gscore","mscore","mbscore","uscore","geoscore","algscore")
exams.to.compute.no.us <- setdiff(exams.to.compute, "uscore")

# set scores above 100 to NA
invisible(lapply(exams.to.compute, function(x) dt.regents.0409[get(x) > 100, (x) := NA]))

build.outcomes <- function(exam.list, suffix = ""){

  # number of attempts
  dt.regents.0409[, (paste0("n_attmpt",suffix)) := sum(as.numeric(!is.na(.SD))), .SDcols = exam.list, by = .(sid,academic.year)]
  dt.regents.0409[, (paste0("ever_attempt",suffix)) := as.numeric(get(paste0("n_attmpt",suffix)) > 0)]
  
  # number pass at 55, 65, and 75
  dt.regents.0409[, (paste0("n_pass55",suffix)) := sum(unlist(lapply(.SD, function(x) as.numeric(x >= 55 & !is.na(x))))), .SDcols = exam.list, by = .(sid, academic.year)]
  dt.regents.0409[, (paste0("n_pass65",suffix)) := sum(unlist(lapply(.SD, function(x) as.numeric(x >= 65 & !is.na(x))))), .SDcols = exam.list, by = .(sid, academic.year)]
  dt.regents.0409[, (paste0("n_pass75",suffix)) := sum(unlist(lapply(.SD, function(x) as.numeric(x >= 75 & !is.na(x))))), .SDcols = exam.list, by = .(sid, academic.year)]
  
  # any pass at 65
  dt.regents.0409[, (paste0("evr_pass65",suffix)) := as.numeric(get(paste0("n_pass65",suffix)) > 0), by = .(sid, academic.year)]
  
  # set n passed to 0 if no exams taken
  dt.regents.0409[get(paste0("ever_attempt", suffix)) == 0, (paste0("n_attmpt", suffix)) := 0]
  dt.regents.0409[get(paste0("ever_attempt", suffix)) == 0, (paste0("n_pass55", suffix)) := 0]
  dt.regents.0409[get(paste0("ever_attempt", suffix)) == 0, (paste0("n_pass65", suffix)) := 0]
  dt.regents.0409[get(paste0("ever_attempt", suffix)) == 0, (paste0("n_pass75", suffix)) := 0]
  
  # fraction of tests taken passed at 55, 65, and 75
  dt.regents.0409[get(paste0("n_attmpt", suffix)) > 0, (paste0("frac_pass55", suffix)) := get(paste0("n_pass55", suffix)) / get(paste0("n_attmpt", suffix))]
  dt.regents.0409[get(paste0("n_attmpt", suffix)) > 0, (paste0("frac_pass65", suffix)) := get(paste0("n_pass65", suffix)) / get(paste0("n_attmpt", suffix))]
  dt.regents.0409[get(paste0("n_attmpt", suffix)) > 0, (paste0("frac_pass75", suffix)) := get(paste0("n_pass75", suffix)) / get(paste0("n_attmpt", suffix))]

}

invisible(build.outcomes(exams.to.compute))
invisible(build.outcomes(exams.to.compute.no.us, "_noUS"))

# z scores
compute.zscore <- function(dt, score.name){
  
  dt[, temp_score := dt[, score.name, with = FALSE]]
  
  # compute standardized score within academic year
  dt[, (paste0("z",score.name)) := (temp_score - mean(temp_score, na.rm = TRUE)) / (var(temp_score,na.rm = TRUE)^(1/2)), by = .(academic.year)]
  
  dt[, temp_score := NULL]
}

invisible(lapply(exams.to.compute, compute.zscore, dt = dt.regents.0409))

# average zscore
dt.regents.0409[, ave_score2zyr := rowMeans(.SD, na.rm = TRUE), .SDcols = paste0("z", exams.to.compute), by =.(sid, academic.year)]
dt.regents.0409[, ave_score2zyr_noUS := rowMeans(.SD, na.rm = TRUE), .SDcols = paste0("z", exams.to.compute.no.us), by =.(sid, academic.year)]

##########################################################################################################
## Lead and Lag Scores

vars.to.lag <- c("ever_attempt", "n_attmpt", "evr_pass65", "n_pass55", "n_pass65", "n_pass75", "ave_score2zyr")

# split data into variables to lead/lag and otherwise to speed year padding
dt.regents.0409.lead.lag <- dt.regents.0409[, c("sid", "academic.year", vars.to.lag), with = FALSE]
dt.regents.0409.other <- dt.regents.0409[, c("sid", "academic.year", setdiff(colnames(dt.regents.0409), colnames(dt.regents.0409.lead.lag))), with = FALSE]

# pad years of data to lead/lag
dt.sid.year <- rbindlist(lapply(2004:2009, function(x)  unique(dt.regents.0409[, .(sid)])[, academic.year := x]))

setkey(dt.sid.year, sid, academic.year)
setkey(dt.regents.0409.lead.lag, sid, academic.year)

dt.regents.0409.lead.lag.pad <- merge(dt.regents.0409.lead.lag, dt.sid.year, by = c("sid", "academic.year"), all = TRUE)

rm("dt.sid.year")
gc()

# for students who did not take exams, fill in ever attempt variable
dt.regents.0409.lead.lag.pad[is.na(ever_attempt), ever_attempt := 0]
dt.regents.0409.lead.lag.pad[ever_attempt == 0, evr_pass65 := 0]
dt.regents.0409.lead.lag.pad[ever_attempt == 0, n_attmpt := 0]
dt.regents.0409.lead.lag.pad[ever_attempt == 0, n_pass55 := 0]
dt.regents.0409.lead.lag.pad[ever_attempt == 0, n_pass65 := 0]
dt.regents.0409.lead.lag.pad[ever_attempt == 0, n_pass75 := 0]

dt.regents.0409.other[is.na(ever_attempt_noUS), ever_attempt_noUS := 0]
dt.regents.0409.other[ever_attempt_noUS == 0, evr_pass65_noUS := 0]
dt.regents.0409.other[ever_attempt_noUS == 0, n_attmpt_noUS := 0]
dt.regents.0409.other[ever_attempt_noUS == 0, n_pass55_noUS := 0]
dt.regents.0409.other[ever_attempt_noUS == 0, n_pass65_noUS := 0]
dt.regents.0409.other[ever_attempt_noUS == 0, n_pass75_noUS := 0]

# lead/lag function
lag.var <- function(dt, var.name, type = "lag", n.lead.lag = 1){

  dt[, temp_var := dt[, var.name, with = FALSE]]
  
  dt[, (paste0(var.name, ".", type, ".", n.lead.lag)) := shift(temp_var, type = type, n = n.lead.lag), by = .(sid)]
  
  dt[, temp_var := NULL]
}

setkey(dt.regents.0409.lead.lag.pad, sid, academic.year)
setDT(dt.regents.0409.lead.lag.pad)

for (tau in 1:3){

  invisible(lapply(vars.to.lag, lag.var, dt = dt.regents.0409.lead.lag.pad, type = "lag", n.lead.lag = tau))
  invisible(lapply(vars.to.lag, lag.var, dt = dt.regents.0409.lead.lag.pad, type = "lead", n.lead.lag = tau))

}

# recombine
dt.regents.0409.out <- merge(dt.regents.0409.lead.lag.pad, dt.regents.0409.other, by = c("sid", "academic.year"), all = TRUE)

rm(list = c("dt.regents.0409.lead.lag", "dt.regents.0409.lead.lag.pad", "dt.regents.0409.other"))
gc()

##########################################################################################################
## Save

saveRDS(dt.regents.0409.out, file = paste0(cleanDir, "dt_regents_scores_0409.Rds"))

rm("dt.regents.0409.out")
gc()