##########################################################################################################
## Program: buildSYEPApp
## Description: builds syep application database, both matched and unmatched
##########################################################################################################

print("Building SYEP application dataset")

##########################################################################################################
## Import

## 2005-2008 SYEP Applications - Matched

raw.05 <- read_dta(paste0(dataDir,"/syepApplications/syep05.dta"))
dt.raw.05.match <- as.data.table(raw.05)
rm("raw.05")

raw.06 <- read_dta(paste0(dataDir,"/syepApplications/syep06.dta"))
dt.raw.06.match <- as.data.table(raw.06)
rm("raw.06")

raw.07 <- read_dta(paste0(dataDir,"/syepApplications/syep07_orig.dta"))
dt.raw.07.match <- as.data.table(raw.07)
rm("raw.07")

raw.08 <- read_dta(paste0(dataDir,"/syepApplications/syep08.dta"))
dt.raw.08.match <- as.data.table(raw.08)
rm("raw.08")

raw.09 <- read_dta(paste0(dataDir,"/syepApplications/syep_2009_matched.dta"))
dt.raw.09.match <- as.data.table(raw.09)
rm("raw.09")

## 2005 - 2008 SYEP Applications - Unmatched

raw.05.nonmatch <- read_dta(paste0(dataDir,"/syepApplications/syep05nonmatch.dta"))
dt.raw.05.nonmatch <- as.data.table(raw.05.nonmatch)
rm("raw.05.nonmatch")

raw.06.nonmatch <- read_dta(paste0(dataDir,"/syepApplications/syep06nonmatch.dta"))
dt.raw.06.nonmatch <- as.data.table(raw.06.nonmatch)
rm("raw.06.nonmatch")

raw.07.nonmatch <- read_dta(paste0(dataDir,"/syepApplications/syep07nonmatch.dta"))
dt.raw.07.nonmatch <- as.data.table(raw.07.nonmatch)
rm("raw.07.nonmatch")

raw.08.nonmatch <- read_dta(paste0(dataDir,"/syepApplications/syep08nonmatch.dta"))
dt.raw.08.nonmatch <- as.data.table(raw.08.nonmatch)
rm("raw.08.nonmatch")

## Import Provider Information

provider.wia.vulnerable <- read_dta(paste0(dataDir,"/auxFiles/provider_wia_vulnerable.dta"))
dt.provider.wia.vulnerable <- as.data.table(provider.wia.vulnerable)
rm("provider.wia.vulnerable")

########################################################################################################
## Combine Matched and Non-Matched Applications

combine.apps <- function(dt.match, dt.nomatch){
  
  dt.match[, matched := 1]
  dt.nomatch[, matched := 0]
  
  dt.nomatch[, sid := -9]
  
  if ("newid" %in% colnames(dt.match)) {
    
    dt.match[, sid := newid]
  
  }
  
  dt.combine <- rbind(dt.match, dt.nomatch, fill = TRUE)
  
  return(dt.combine)
}

dt.raw.05 <- combine.apps(dt.raw.05.match, dt.raw.05.nonmatch)
dt.raw.06 <- combine.apps(dt.raw.06.match, dt.raw.06.nonmatch)
dt.raw.07 <- combine.apps(dt.raw.07.match, dt.raw.07.nonmatch)
dt.raw.08 <- combine.apps(dt.raw.08.match, dt.raw.08.nonmatch)

# no non-matched 2009 files
dt.raw.09 <- dt.raw.09.match
dt.raw.09[, matched := 1]

rm(list = c("dt.raw.09.match", paste0(c("dt.raw.05","dt.raw.06","dt.raw.07","dt.raw.08"), ".match"), 
  paste0(c("dt.raw.05","dt.raw.06","dt.raw.07","dt.raw.08"), ".nonmatch")))
gc()

########################################################################################################
## Cleaning

## Fill in Missing Years

dt.raw.05[is.na(syepyear), syepyear := 2005]
dt.raw.06[is.na(syepyear), syepyear := 2006]
dt.raw.07[is.na(syepyear), syepyear := 2007]
dt.raw.08[is.na(syepyear), syepyear := 2008]
dt.raw.09[is.na(syepyear), syepyear := 2009]

## Column Name Consolidation

dt.raw.09[, hours := hoursworked]
dt.raw.09[, provider := providername]
dt.raw.09[, selbylot := selectedbylottery]

## Clean ID and Drop Missing

dt.raw.05[, sid := as.numeric(sid)]
dt.raw.07[, sid := as.numeric(sid)]
dt.raw.06[, sid := as.numeric(sid)]
dt.raw.08[, sid := as.numeric(sid)]
dt.raw.09[, sid := as.numeric(newid)]

dt.syep.stack <- rbindlist(list(dt.raw.05, dt.raw.06, dt.raw.07, dt.raw.08, dt.raw.09), fill = TRUE)

dt.syep.nonmiss <- dt.syep.stack[!is.na(sid) | matched == 0]

dt.syep.nonmiss[, syepyear := as.numeric(syepyear)]

# negate matched so sorting puts matched first
dt.syep.nonmiss[, matched.sort := -matched]

setkey(dt.syep.nonmiss, matched.sort, sid, syepyear)

# assign row ID to unmatched
dt.syep.nonmiss[matched == 0, unmatched.id := .I]

## Flag Selection and Participation

dt.syep.nonmiss[, hours := as.numeric(hours)]

dt.syep.nonmiss[, worked := as.numeric(hours > 0 & !is.na(hours))]
dt.syep.nonmiss[, select := as.numeric(selbylot == "Y")]

## Scrub Provider Name of Punctuation and Spaces (uppercase as well)

dt.syep.nonmiss[, provider.clean := toupper(gsub("[[:punct:] [:space:]]", "", provider))]
dt.provider.wia.vulnerable[, provider.clean := toupper(gsub("[[:punct:] [:space:]]", "", provider))]

# sub out unicode
dt.syep.nonmiss[, provider.clean := iconv(provider.clean,"UTF-8", "ASCII", sub = "")]

## Provider Drops

# merge in provider information
dt.syep.nonmiss <- merge(dt.syep.nonmiss, dt.provider.wia.vulnerable[, .(provider.clean, wia,vulnerable)], by = c("provider.clean"), all.x = TRUE)

# flag WIA and vulnerable to drop
dt.syep.nonmiss[, wia.vul.drop := as.numeric(wia + vulnerable > 0)]

# if provider is not listed, do not drop
dt.syep.nonmiss[is.na(wia.vul.drop), wia.vul.drop := 0]

# flag select providers for drop
provider.drop <- c("National Society For Hebrew Day Schools - Out of C", "National Society For Hebrew Day Schools - Gen Pop",
  "Henry Street Settlement - Out of City", "COJO Flatbush - Out of City", "Caribbean & American Family Services - Out of City")

provider.clean.drop <- toupper(gsub("[[:punct:] [:space:]]", "", provider.drop))

dt.syep.nonmiss[, provider.drop := as.numeric(provider.clean %in% provider.clean.drop)]

## Compute Win Rate within Lottery

# flag whether to use unmatched as well to compute win rates
lottery.win.rate.use.nonmatch <- FALSE

if (lottery.win.rate.use.nonmatch) {
  
  dt.syep.nonmiss[, pctsel := mean(select), by = .(syepyear, provider.clean)]

} else {

  dt.syep.nonmiss[, pctsel := mean(select[matched == 1]), by = .(syepyear, provider.clean)]

}

# drop lotteries with greater than 99% or 0% selection
dt.syep.nonmiss[, pctsel.drop := as.numeric(pctsel > .99 | pctsel == 0)]

## Duplicate Drops

# retain unmatched and unique matched application entries only
dt.syep.unique <- unique(dt.syep.nonmiss[, .(sid, matched, unmatched.id, syepyear, provider.clean, provider.drop, 
  worked, select, wia, vulnerable, wia.vul.drop, pctsel, pctsel.drop, lastgrd, seqno)])

# flag duplicates within a year
dt.syep.unique[matched == 1, count.within.year :=.N, by = .(sid, syepyear)]
dt.syep.unique[matched == 1, has.dupe.app := as.numeric(count.within.year > 1)]

# within year, sum selections and participation for students with multiple applications
dt.syep.unique[has.dupe.app == 1, total.select.dupe := sum(select), by = .(sid, syepyear)]
dt.syep.unique[has.dupe.app == 1, total.worked.dupe := sum(worked), by = .(sid, syepyear)]

# within a year, sort duplicates by selection and working. keep first duplicate for merging into student files
setorder(dt.syep.unique, -matched, sid, syepyear, provider.drop, -select, -worked, pctsel, provider.clean)

dt.syep.unique[has.dupe.app == 1, dupe.keep := as.numeric(seq(.N) == 1), by = .(sid, syepyear)]

dt.syep.unique[is.na(dupe.keep), dupe.keep := 0]

## Aggregate Drop Flags

dt.syep.unique[matched == 0, has.dupe.app := 0]

dt.syep.unique[, app.drop.flag := as.numeric(has.dupe.app + wia.vul.drop + provider.drop + pctsel.drop > 0)]

## Misc Cleaning and Variable Creation

# grade of application fixed effect
dt.syep.unique[, lastgrd.numeric := as.numeric(lastgrd)]
dt.syep.unique[!lastgrd.numeric %in% 7:12, lastgrd.numeric:=0]
dt.syep.unique[, app.fe.grade := as.factor(lastgrd.numeric)]

# lottery fixed effect
dt.syep.unique[, app.fe.lottery := as.factor(paste0(syepyear, provider.clean))]

# flag first application (if matched)
dt.syep.unique[matched == 1, first.app.flag.app.raw := as.numeric(syepyear == min(syepyear)), by = .(sid)]
dt.syep.unique[matched == 0, first.app.flag.app.raw := 1]

##########################################################################################################
## Save

saveRDS(dt.syep.unique, file = paste0(cleanDir,"/dt_syep_applications.Rds"))

rm("dt.syep.unique")
gc()