##########################################################################################################
## Program: buildStudentFiles
## Description: builds file of student covariates, keyed by student and year
##########################################################################################################

print("Building student covariate dataset")

##########################################################################################################
## NOTES

## .dta are converted to .Rds and saved, if user does not wish to re-read in .dta, set dt.flag == 0

dta.flag <- 0

##########################################################################################################
## Import

if (dta.flag == 1){
  
  ## 2004/2005 - 2008/2009 Student Files

  mrs.0405 <- read_dta(paste0(dataDir,"/studentFiles/MRS_SYEP_y0405x_cleaned.dta"))
  dt.mrs.0405 <- as.data.table(mrs.0405)
  saveRDS(dt.mrs.0405, file = paste0(cleanDir, "/dt_mrs_student_0405.Rds"))
  rm(list = c("mrs.0405", "dt.mrs.0405"))
  
  mrs.0506 <- read_dta(paste0(dataDir, "/studentFiles/MRS_SYEP_y0506x_cleaned.dta"))
  dt.mrs.0506 <- as.data.table(mrs.0506)
  saveRDS(dt.mrs.0506, file = paste0(cleanDir, "/dt_mrs_student_0506.Rds"))
  rm(list = c("mrs.0506", "dt.mrs.0506"))
  
  mrs.0607 <- read_dta(paste0(dataDir, "/studentFiles/MRS_SYEP_y0607x_cleaned.dta"))
  dt.mrs.0607 <- as.data.table(mrs.0607)
  saveRDS(dt.mrs.0607, file = paste0(cleanDir, "/dt_mrs_student_0607.Rds"))
  rm(list = c("mrs.0607", "dt.mrs.0607"))
  
  mrs.0708 <- read_dta(paste0(dataDir, "/studentFiles/MRS_SYEP_y0708x_cleaned.dta"))
  dt.mrs.0708 <- as.data.table(mrs.0708)
  saveRDS(dt.mrs.0708, file = paste0(cleanDir, "/dt_mrs_student_0708.Rds"))
  rm(list = c("mrs.0708", "dt.mrs.0708"))
  
  mrs.0809 <- read_dta(paste0(dataDir, "/studentFiles/MRS_SYEP_y0809x_cleaned.dta"))
  dt.mrs.0809 <- as.data.table(mrs.0809)
  saveRDS(dt.mrs.0809, file = paste0(cleanDir, "/dt_mrs_student_0809.Rds"))
  rm(list = c("mrs.0809", "dt.mrs.0809"))
  
  ## 8th Grade Test Scores
  
  scores.8th <- read_dta(paste0(dataDir, "/auxFiles/eighth grade test scores.dta"))
  dt.scores.8th <- as.data.table(scores.8th)
  saveRDS(dt.scores.8th, file = paste0(cleanDir, "/scores_8th_dt.Rds"))
  rm(list = c("scores.8th"))
  
}

########################################################################################################
## Cleaning by Year

clean.mrs.files <- function(dt){
  
  ## Fill in Missing Years
  
  dt[, year := max.mode(year)]
  
  ## Student ID
  
  # retain nonmissing student ids
  if ( "newid" %in% colnames(dt) & !"sid" %in% colnames(dt)) {
    
    dt.nonmiss <- dt[newid != ""]
    
    dt.nonmiss[, sid := as.numeric(newid)]
  
  } else {
  
    dt.nonmiss <- dt[!is.na(sid)]
  
  }

  ## Create District
  
  dt.nonmiss[, bds := bdsoct]
  dt.nonmiss[bds == "", bds := bdsmar]
  dt.nonmiss[bds == "", bds := bdsjun]
  
  dt.nonmiss[, boro := substr(bds, 1, 1)]
  dt.nonmiss[boro == "1", boro := "M"]
  dt.nonmiss[boro == "2", boro := "X"]
  dt.nonmiss[boro == "3", boro := "K"]
  dt.nonmiss[boro == "4", boro := "Q"]
  dt.nonmiss[boro == "5", boro := "R"]
  
  dt.nonmiss[, district := substr(bds, 2, 3)]
  dt.nonmiss[, sch := substr(bds, 4, 3)]
  dt.nonmiss[, dbn := paste0(district, boro, sch)]
  
  # flag districts to drop
  dt.nonmiss[, district.drop := as.numeric(district %in% c(75,76))]
  
  ## Demographics
  
  # gender
  if ( "sex" %in% colnames(dt.nonmiss)) {
    
    dt.nonmiss[, female := as.numeric(sex == "F")]
    dt.nonmiss[, miss_gender := as.numeric(sex == "")]
  
  } else if ("gender" %in% colnames(dt.nonmiss)) {
  
    dt.nonmiss[, female := as.numeric(gender == "F")]
    dt.nonmiss[, miss_gender := as.numeric(gender == "")]
  
  }
  
  # race
  if ("ethnic2" %in% colnames(dt.nonmiss)) {
    
    dt.nonmiss[, asian := as.numeric(ethnic2 == 2 & !is.na(ethnic2))]
    dt.nonmiss[, hispanic := as.numeric(ethnic2 == 3 & !is.na(ethnic2))]
    dt.nonmiss[, black := as.numeric(ethnic2 == 4 & !is.na(ethnic2))]
    dt.nonmiss[, white := as.numeric(ethnic2 == 5 & !is.na(ethnic2))]
    
    dt.nonmiss[, miss_ethn := as.numeric(is.na(ethnic2))]
  
  }
  
  # free and reduced lunch
  if ("mealcodex" %in% colnames(dt.nonmiss)) {
    
    dt.nonmiss[, freelu := as.numeric(mealcodex == 1 & !is.na(mealcodex))]
    dt.nonmiss[, redlu := as.numeric(mealcodex == 2 & !is.na(mealcodex))]
    dt.nonmiss[, fulllu := as.numeric(mealcodex == 3 & !is.na(mealcodex))]
    
    dt.nonmiss[, missfrl := as.numeric(is.na(mealcodex))]
  
  }

  ## Grade and Cohort
  
  # grade (fill in missing grade with grade2)
  dt.nonmiss[, grade := grade2]
  dt.nonmiss[, grade_miss := as.numeric(is.na(grade))]
  
  ## Retain Relevant Variables

  cols.to.keep <- intersect(
    colnames(dt.nonmiss), 
    c("sid","year","dob_sas","grade","grade_miss","female","miss_gender","white","black","hispanic","asian","ethnic2", "miss_ethn","native","freelu","redlu","missfrl","univfeed","mealcodex", 
      "lep","esl","sped","zread","zmath","exclude","district.drop","dbn")
      )
  
  dt.out <- dt.nonmiss[!is.na(sid), cols.to.keep, with = FALSE]
  
  rm("dt.nonmiss")
  gc()
  
  setkey(dt.out, sid)
  
  return(dt.out)
}

student.files.clean.list <- list()
for (year.num in seq(4, 8)) {
  
  year.suffix <- as.character(paste0("0", year.num, "0", year.num + 1))
  
  print(paste("Importing and Cleaning Year", year.suffix))
  print(paste0("Path: ", cleanDir, "dt_mrs_student_", year.suffix, ".Rds"))
  
  dt.raw.mrs <- readRDS(paste0(cleanDir, "dt_mrs_student_", year.suffix, ".Rds"))
  
  student.files.clean.list[[length(student.files.clean.list) + 1]] <- clean.mrs.files(dt.raw.mrs)
  
  rm("dt.raw.mrs")
  gc()

}

dt.clean.stack <- rbindlist(student.files.clean.list, fill = TRUE)

setkey(dt.clean.stack, sid, year)

########################################################################################################
## Cleaning

dt.clean <- dt.clean.stack

rm(list = c("dt.clean.stack", "student.files.clean.list"))
gc()

setkey(dt.clean, sid, year)

# estimate age at SYEP receipt for non-applicants assuming SYEP starts June 1
dt.clean[, age.synth := time_length(interval(dob_sas, as.Date(paste0(year, "-06-01"))), "years")]
dt.clean[, age_syep_rec := age.synth]

# assign 0 to missing age and flag missing age
dt.clean[, age_flag := as.numeric(is.na(age_syep_rec))]
dt.clean[is.na(age_syep_rec), age_syep_rec := 0]

## Variable Creation

# esl but not lep variable
dt.clean[, esl.not.lep := as.numeric(esl == 1 & lep == 0)]

## Modal Characteristic Creation

# gender
dt.clean[, female.mode := max.mode(female), by = .(sid)]

dt.clean[, miss_gender.mode := max.mode(miss_gender)]

# race
dt.clean[, ethnic2.mode := max.mode(ethnic2), by = .(sid)]

dt.clean[, asian.mode := as.numeric(ethnic2.mode == 2 & !is.na(ethnic2.mode))]
dt.clean[, hispanic.mode := as.numeric(ethnic2.mode == 3 & !is.na(ethnic2.mode))]
dt.clean[, black.mode := as.numeric(ethnic2.mode == 4 & !is.na(ethnic2.mode))]
dt.clean[, white.mode := as.numeric(ethnic2.mode == 5 & !is.na(ethnic2.mode))]

dt.clean[, miss_ethn.mode := as.numeric(is.na(ethnic2.mode))]

# meal code (including override with univfeed)
dt.clean[, mealcodex.mode := max.mode(mealcodex), by = .(sid)]
dt.clean[is.na(mealcodex.mode) & univfeed == "Y", mealcodex.mode := 1]
dt.clean[, mealcodex.mode := max.mode(mealcodex.mode), by = .(sid)] # re-doing maxmode to capture corrections

dt.clean[, freelu.mode := as.numeric(mealcodex.mode == 1 & !is.na(mealcodex.mode))]
dt.clean[, redlu.mode := as.numeric(mealcodex.mode == 2 & !is.na(mealcodex.mode))]
dt.clean[, fulllu.mode := as.numeric(mealcodex.mode == 3 & !is.na(mealcodex.mode))]

dt.clean[, missfrl.mode := as.numeric(is.na(mealcodex.mode))]

# LEP, ESL, and Spec Ed
dt.clean[, lep.mode := max.mode(lep), by = .(sid)]
dt.clean[, esl.mode:=max.mode(esl), by = .(sid)]
dt.clean[, esl.not.lep.mode:=max.mode(esl.not.lep), by = .(sid)]
dt.clean[, sped.mode := max.mode(sped), by = .(sid)]

# native
dt.clean[, native.mode := max.mode(native), by = .(sid)]

## Cohorts

# set grade 96 to 97
dt.clean[grade == 96, grade := 97]

# cohort is year in which student is given grade
dt.clean[grade == 8, cohort8_year := year]
dt.clean[grade == 9, cohort9_year := year]
dt.clean[grade == 10, cohort10_year := year]
dt.clean[grade == 11, cohort11_year := year]
dt.clean[grade == 12, cohort12_year := year]

# take first cohort year if repeated grades
dt.clean[, cohort8_year := min(cohort8_year, 3000, na.rm = TRUE), by = .(sid)]
dt.clean[, cohort9_year := min(cohort9_year, 3000, na.rm = TRUE), by = .(sid)]
dt.clean[, cohort10_year := min(cohort10_year, 3000, na.rm = TRUE), by = .(sid)]
dt.clean[, cohort11_year := min(cohort11_year, 3000, na.rm = TRUE), by = .(sid)]
dt.clean[, cohort12_year := min(cohort12_year, 3000, na.rm = TRUE), by = .(sid)]

dt.clean[cohort8_year == 3000, cohort8_year := NA]
dt.clean[cohort9_year == 3000, cohort9_year := NA]
dt.clean[cohort10_year == 3000, cohort10_year := NA]
dt.clean[cohort11_year == 3000, cohort11_year := NA]
dt.clean[cohort12_year == 3000, cohort12_year := NA]

# impute cohort 9 if missing
dt.clean[is.na(cohort9_year), cohort9_year := cohort10_year - 1, by = .(sid)]
dt.clean[is.na(cohort9_year), cohort9_year := cohort11_year - 2, by = .(sid)]
dt.clean[is.na(cohort9_year), cohort9_year := cohort12_year - 3, by = .(sid)]
dt.clean[is.na(cohort9_year), cohort9_year := cohort8_year + 1, by = .(sid)]

# grade dummies
dt.clean[, grade8 := as.numeric(grade == 8)]
dt.clean[, grade9 := as.numeric(grade == 9)]
dt.clean[, grade10 := as.numeric(grade == 10)]
dt.clean[, grade11 := as.numeric(grade == 11)]
dt.clean[, grade97 := as.numeric(grade == 97)]

dt.clean[is.na(grade), grade8 := 0]
dt.clean[is.na(grade), grade9 := 0]
dt.clean[is.na(grade), grade10 := 0]
dt.clean[is.na(grade), grade11 := 0]
dt.clean[is.na(grade), grade97 := 0]

## 8th Grade Test Scores

dt.scores.8th <- readRDS(paste0(cleanDir,"/scores_8th_dt.Rds"))
dt.scores.8th[, zmath8 := zmath]
dt.scores.8th[, zread8 := zread]

dt.clean.scores <- merge(dt.clean, dt.scores.8th[,.(sid, zmath8, zread8, year_eighth)], by = c("sid"), all.x = TRUE)

# set missing values to 0 and flag
dt.clean.scores[,zread8_flag := as.numeric(is.na(zread8))]
dt.clean.scores[,zmath8_flag := as.numeric(is.na(zmath8))]

dt.clean.scores[is.na(zread8), zread8_rec := 0]
dt.clean.scores[!is.na(zread8), zread8_rec := zread8]
dt.clean.scores[is.na(zmath8), zmath8_rec := 0]
dt.clean.scores[!is.na(zmath8), zmath8_rec := zmath8]

##########################################################################################################
## Save

saveRDS(dt.clean.scores, file = paste0(cleanDir, "/dt_student_files.Rds"))

rm("dt.clean.scores")
gc()